@@ -80,8 +80,20 @@ struct
   unsigned long max_cache_ttl;     /* Default. */
   unsigned long max_cache_ttl_ssh; /* for SSH. */
 
+  /* Flag disallowin bypassing of the warning.  */
+  int enforce_passphrase_constraints;
   /* The require minmum length of a passphrase. */
   unsigned int min_passphrase_len;
+  /* The minimum number of non-alpha characters in a passphrase.  */
+  unsigned int min_passphrase_nonalpha;
+  /* File name with a patternfile or NULL if not enabled.  */
+  const char *check_passphrase_pattern;
+  /* If not 0 the user is asked to change his passphrase after these
+     number of days.  */
+  unsigned int max_passphrase_days;
+  /* If set, a passphrase history will be written and checked at each
+     passphrase change.  */
+  int enable_passhrase_history;
 
   int running_detached; /* We are running detached from the tty. */
 
@@ -146,6 +158,8 @@ struct server_control_s
 
   int use_auth_call; /* Hack to send the PKAUTH command instead of the
                         PKSIGN command to the scdaemon.  */
+  int in_passwd;     /* Hack to inhibit enforced passphrase change
+                        during an explicit passwd command.  */
 };
 
 
@@ -175,7 +189,7 @@ enum
 /* Values for the cache_mode arguments. */
 typedef enum 
   {
-    CACHE_MODE_IGNORE = 0, /* Special mode to by pass the cache. */
+    CACHE_MODE_IGNORE = 0, /* Special mode to bypass the cache. */
     CACHE_MODE_ANY,        /* Any mode except ignore matches. */
     CACHE_MODE_NORMAL,     /* Normal cache (gpg-agent). */
     CACHE_MODE_USER,       /* GET_PASSPHRASE related cache. */
@@ -227,6 +241,7 @@ int agent_get_passphrase (ctrl_t ctrl, char **retpass,
                           const char *errtext);
 int agent_get_confirmation (ctrl_t ctrl, const char *desc, const char *ok,
 			    const char *cancel);
+int agent_show_message (ctrl_t ctrl, const char *desc, const char *ok_btn);
 int agent_popup_message_start (ctrl_t ctrl,
                                const char *desc, const char *ok_btn);
 void agent_popup_message_stop (ctrl_t ctrl);
@@ -263,6 +278,7 @@ int agent_protect_and_store (ctrl_t ctrl, gcry_sexp_t s_skey);
 int agent_protect (const unsigned char *plainkey, const char *passphrase,
                    unsigned char **result, size_t *resultlen);
 int agent_unprotect (const unsigned char *protectedkey, const char *passphrase,
+                     gnupg_isotime_t protected_at, 
                      unsigned char **result, size_t *resultlen);
 int agent_private_key_type (const unsigned char *privatekey);
 unsigned char *make_shadow_info (const char *serialno, const char *idstring);
