@@ -1,12 +1,12 @@
 /* passphrase.c -  Get a passphrase
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
- *               2005, 2006 Free Software Foundation, Inc.
+ *               2005, 2006, 2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -259,7 +257,9 @@ passphrase_get ( u32 *keyid, int mode, const char *cacheid,
   PKT_public_key *pk = xmalloc_clear( sizeof *pk );
   byte fpr[MAX_FINGERPRINT_LEN];
   int have_fpr = 0;
+#ifdef ENABLE_NLS
   char *orig_codeset = NULL;
+#endif
   char *my_prompt;
   char hexfprbuf[20*2+1];
   const char *my_cacheid;
@@ -281,7 +281,7 @@ passphrase_get ( u32 *keyid, int mode, const char *cacheid,
   
 #ifdef ENABLE_NLS
   /* The Assuan agent protocol requires us to transmit utf-8 strings */
-  orig_codeset = bind_textdomain_codeset (PACKAGE, NULL);
+  orig_codeset = bind_textdomain_codeset (PACKAGE_GT, NULL);
 #ifdef HAVE_LANGINFO_CODESET
   if (!orig_codeset)
     orig_codeset = nl_langinfo (CODESET);
@@ -289,8 +289,11 @@ passphrase_get ( u32 *keyid, int mode, const char *cacheid,
   if (orig_codeset)
     { /* We only switch when we are able to restore the codeset later. */
       orig_codeset = xstrdup (orig_codeset);
-      if (!bind_textdomain_codeset (PACKAGE, "utf-8"))
-        orig_codeset = NULL; 
+      if (!bind_textdomain_codeset (PACKAGE_GT, "utf-8"))
+        {
+	  xfree (orig_codeset);
+	  orig_codeset = NULL; 
+	}
     }
 #endif
 
@@ -393,7 +396,7 @@ passphrase_get ( u32 *keyid, int mode, const char *cacheid,
 #ifdef ENABLE_NLS
   if (orig_codeset)
     {
-      bind_textdomain_codeset (PACKAGE, orig_codeset);
+      bind_textdomain_codeset (PACKAGE_GT, orig_codeset);
       xfree (orig_codeset);
     }
 #endif
