@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -200,7 +198,7 @@ int
 gen_desig_revoke( const char *uname, strlist_t locusr )
 {
     int rc = 0;
-    armor_filter_context_t afx;
+    armor_filter_context_t *afx;
     PKT_public_key *pk = NULL;
     PKT_secret_key *sk = NULL;
     PKT_signature *sig = NULL;
@@ -219,7 +217,7 @@ gen_desig_revoke( const char *uname, strlist_t locusr )
 	return G10ERR_GENERAL;
       }
 
-    memset( &afx, 0, sizeof afx);
+    afx = new_armor_context ();
 
     kdbhd = keydb_new (0);
     classify_user_id (uname, &desc);
@@ -331,10 +329,10 @@ gen_desig_revoke( const char *uname, strlist_t locusr )
 	    if( (rc = open_outfile( NULL, 0, &out )) )
 	      goto leave;
 
-	    afx.what = 1;
-	    afx.hdrlines = "Comment: A designated revocation certificate"
+	    afx->what = 1;
+	    afx->hdrlines = "Comment: A designated revocation certificate"
 	      " should follow\n";
-	    iobuf_push_filter( out, armor_filter, &afx );
+	    push_armor_filter (afx, out);
 
 	    /* create it */
 	    rc = make_keysig_packet( &sig, pk, NULL, NULL, sk, 0x20, 0,
@@ -427,6 +425,7 @@ gen_desig_revoke( const char *uname, strlist_t locusr )
     else
 	iobuf_close(out);
     release_revocation_reason_info( reason );
+    release_armor_context (afx);
     return rc;
 }
 
@@ -438,7 +437,7 @@ int
 gen_revoke( const char *uname )
 {
     int rc = 0;
-    armor_filter_context_t afx;
+    armor_filter_context_t *afx;
     PACKET pkt;
     PKT_secret_key *sk; /* used as pointer into a kbnode */
     PKT_public_key *pk = NULL;
@@ -457,7 +456,7 @@ gen_revoke( const char *uname )
 	return G10ERR_GENERAL;
       }
 
-    memset( &afx, 0, sizeof afx);
+    afx = new_armor_context ();
     init_packet( &pkt );
 
     /* search the userid: 
@@ -556,9 +555,9 @@ gen_revoke( const char *uname )
     if( (rc = open_outfile( NULL, 0, &out )) )
 	goto leave;
 
-    afx.what = 1;
-    afx.hdrlines = "Comment: A revocation certificate should follow\n";
-    iobuf_push_filter( out, armor_filter, &afx );
+    afx->what = 1;
+    afx->hdrlines = "Comment: A revocation certificate should follow\n";
+    push_armor_filter (afx, out);
 
     /* create it */
     rc = make_keysig_packet( &sig, pk, NULL, NULL, sk, 0x20, 0,
@@ -609,6 +608,7 @@ gen_revoke( const char *uname )
     else
 	iobuf_close(out);
     release_revocation_reason_info( reason );
+    release_armor_context (afx);
     return rc;
 }
 
