@@ -338,11 +338,14 @@ make_filename( const char *first_part, ... )
 }
 
 
+/* Compare whether the filenames are identical.  This is a
+   special version of strcmp() taking the semantics of filenames in
+   account.  Note that this function works only on the supplied names
+   without considereing any context like the current directory.  See
+   also same_file_p(). */
 int
 compare_filenames (const char *a, const char *b)
 {
-  /* ? check whether this is an absolute filename and resolve
-     symlinks?  */
 #ifdef HAVE_DRIVE_LETTERS
   for ( ; *a && *b; a++, b++ ) 
     {
