@@ -1,5 +1,5 @@
-/* export.c
- * Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
+/* export.c - Export certificates and private keys.
+ * Copyright (C) 2002, 2003, 2004, 2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -32,6 +32,7 @@
 #include "keydb.h"
 #include "exechelp.h"
 #include "i18n.h"
+#include "sysutils.h"
 
 
 
@@ -378,10 +379,24 @@ gpgsm_p12_export (ctrl_t ctrl, const char *name, FILE *fp)
           log_error ("keydb_get_cert failed: %s\n", gpg_strerror (rc));
           goto leave;
         }
-      
+
+    next_ambiguous:      
       rc = keydb_search (hd, desc, 1);
       if (!rc)
-        rc = gpg_error (GPG_ERR_AMBIGUOUS_NAME);
+        {
+          ksba_cert_t cert2 = NULL;
+
+          if (!keydb_get_cert (hd, &cert2))
+            {
+              if (gpgsm_certs_identical_p (cert, cert2))
+                {
+                  ksba_cert_release (cert2);
+                  goto next_ambiguous;
+                }
+              ksba_cert_release (cert2);
+            }
+          rc = gpg_error (GPG_ERR_AMBIGUOUS_NAME);
+        }
       else if (rc == -1 || gpg_err_code (rc) == GPG_ERR_EOF)
         rc = 0;
       if (rc)
@@ -606,7 +621,7 @@ export_p12 (ctrl_t ctrl, const unsigned char *certimg, size_t certimglen,
   else
     pgmname = opt.protect_tool_program;
 
-  infp = tmpfile ();
+  infp = gnupg_tmpfile ();
   if (!infp)
     {
       err = gpg_error_from_syserror ();
@@ -622,7 +637,7 @@ export_p12 (ctrl_t ctrl, const unsigned char *certimg, size_t certimglen,
       goto cleanup;
     }
 
-  outfp = tmpfile ();
+  outfp = gnupg_tmpfile ();
   if (!outfp)
     {
       err = gpg_error_from_syserror ();
@@ -698,7 +713,7 @@ export_p12 (ctrl_t ctrl, const unsigned char *certimg, size_t certimglen,
     fclose (fp);
   if (pid != -1)
     {
-      if (!gnupg_wait_process (pgmname, pid))
+      if (!gnupg_wait_process (pgmname, pid, NULL))
         child_err = 0;
     }
   if (!err)
