@@ -31,6 +31,7 @@
 #include <time.h>
 #include <stdarg.h>
 #include <signal.h>
+#include <ctype.h>
 #ifdef HAVE_W32_SYSTEM
 # define WIN32_LEAN_AND_MEAN 1
 # include <windows.h>
@@ -43,6 +44,7 @@
 #define JNLIB_NEED_LOG_LOGV
 #include "util.h"
 #include "i18n.h"
+#include "exechelp.h"
 
 #include "gc-opt-flags.h"
 #include "gpgconf.h"
@@ -153,6 +155,12 @@ static struct
      GPGConf.  In this case, PROGRAM is NULL.  */
   char *program;
 
+  /* The module name (GNUPG_MODULE_NAME_foo) as defined by
+     ../common/util.h.  This value is used to get the actual installed
+     path of the program.  0 is used if no backedn program is
+     available. */
+  char module_name;
+
   /* The runtime change callback.  */
   void (*runtime_change) (void);
 
@@ -168,14 +176,18 @@ static struct
 } gc_backend[GC_BACKEND_NR] =
   {
     { NULL },		/* GC_BACKEND_ANY dummy entry.  */
-    { "GnuPG", GPGNAME, NULL, "gpgconf-gpg.conf" },
-    { "GPGSM", "gpgsm", NULL, "gpgconf-gpgsm.conf" },
-    { "GPG Agent", "gpg-agent", gpg_agent_runtime_change,
-      "gpgconf-gpg-agent.conf" },
-    { "SCDaemon", "scdaemon", NULL, "gpgconf-scdaemon.conf" },
-    { "DirMngr", "dirmngr", NULL, "gpgconf-dirmngr.conf" },
-    { "DirMngr LDAP Server List", NULL, NULL, "ldapserverlist-file",
-      "LDAP Server" },
+    { "GnuPG", GPGNAME, GNUPG_MODULE_NAME_GPG,
+      NULL, "gpgconf-gpg.conf" },
+    { "GPGSM", "gpgsm", GNUPG_MODULE_NAME_GPGSM,
+      NULL, "gpgconf-gpgsm.conf" },
+    { "GPG Agent", "gpg-agent", GNUPG_MODULE_NAME_AGENT, 
+      gpg_agent_runtime_change, "gpgconf-gpg-agent.conf" },
+    { "SCDaemon", "scdaemon", GNUPG_MODULE_NAME_SCDAEMON,
+      NULL, "gpgconf-scdaemon.conf" },
+    { "DirMngr", "dirmngr", GNUPG_MODULE_NAME_DIRMNGR,
+      NULL, "gpgconf-dirmngr.conf" },
+    { "DirMngr LDAP Server List", NULL, 0, 
+      NULL, "ldapserverlist-file", "LDAP Server" },
   };
 
 
@@ -505,14 +517,38 @@ static gc_option_t gc_options_gpg_agent[] =
    { "allow-mark-trusted", GC_OPT_FLAG_RUNTIME,
      GC_LEVEL_ADVANCED, "gnupg", "allow clients to mark keys as \"trusted\"",
      GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
-   { "min-passphrase-len", GC_OPT_FLAG_RUNTIME,
-     GC_LEVEL_EXPERT, "gnupg", 
-     N_("|N|set minimal required length for new passphrases to N"),
-     GC_ARG_TYPE_UINT32, GC_BACKEND_GPG_AGENT },
    { "no-grab", GC_OPT_FLAG_RUNTIME, GC_LEVEL_EXPERT,
      "gnupg", "do not grab keyboard and mouse",
      GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
 
+   { "Passphrase policy",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Options enforcing a passphrase policy") },
+   { "enforce-passphrases-constraints", GC_OPT_FLAG_RUNTIME, 
+     GC_LEVEL_EXPERT, "gnupg", 
+     N_("do not allow to bypass the passphrase policy"),
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+   { "min-passphrase-len", GC_OPT_FLAG_RUNTIME,
+     GC_LEVEL_ADVANCED, "gnupg", 
+     N_("|N|set minimal required length for new passphrases to N"),
+     GC_ARG_TYPE_UINT32, GC_BACKEND_GPG_AGENT },
+   { "min-passphrase-nonalpha", GC_OPT_FLAG_RUNTIME,
+     GC_LEVEL_EXPERT, "gnupg", 
+     N_("|N|require at least N non-alpha characters for a new passphrase"),
+     GC_ARG_TYPE_UINT32, GC_BACKEND_GPG_AGENT },
+   { "check-passphrase-pattern", GC_OPT_FLAG_RUNTIME,
+     GC_LEVEL_EXPERT,
+     "gnupg", N_("|FILE|check new passphrases against pattern in FILE"),
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_SCDAEMON },
+   { "max-passphrase-days", GC_OPT_FLAG_RUNTIME,
+     GC_LEVEL_EXPERT, "gnupg", 
+     N_("|N|expire the passphrase after N days"),
+     GC_ARG_TYPE_UINT32, GC_BACKEND_GPG_AGENT },
+   { "enable-passphrases-history", GC_OPT_FLAG_RUNTIME, 
+     GC_LEVEL_EXPERT, "gnupg", 
+     N_("do not allow the reuse of old passphrases"),
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+
    GC_OPTION_NULL
  };
 
@@ -915,11 +951,24 @@ static struct
     { "gpg-agent", NULL, "GPG Agent", gc_options_gpg_agent },
     { "scdaemon", NULL, "Smartcard Daemon", gc_options_scdaemon },
     { "gpgsm", NULL, "GPG for S/MIME", gc_options_gpgsm },
-#ifndef HAVE_W32_SYSTEM
     { "dirmngr", NULL, "Directory Manager", gc_options_dirmngr }
-#endif
   };
 
+
+
+/* Structure used to collect error output of the backend programs.  */
+struct error_line_s;
+typedef struct error_line_s *error_line_t;
+struct error_line_s
+{
+  error_line_t next;   /* Link to next item.  */
+  const char *fname;   /* Name of the config file (points into BUFFER).  */
+  unsigned int lineno; /* Line number of the config file.  */
+  const char *errtext; /* Text of the error message (points into BUFFER).  */
+  char buffer[1];  /* Helper buffer.  */
+};
+
+
 
 /* Engine specific support.  */
 void
@@ -1093,20 +1142,264 @@ percent_deescape (const char *src)
 void
 gc_component_list_components (FILE *out)
 {
-  gc_component_t idx;
+  gc_component_t component;
+  gc_option_t *option;
+  gc_backend_t backend;
+  int backend_seen[GC_BACKEND_NR];
+  const char *desc;
+  const char *pgmname;
 
-  for (idx = 0; idx < GC_COMPONENT_NR; idx++)
+  for (component = 0; component < GC_COMPONENT_NR; component++)
     {
-      if (gc_component[idx].options)
+      option = gc_component[component].options;
+      if (option)
         {
-          const char *desc = gc_component[idx].desc;
-          desc = my_dgettext (gc_component[idx].desc_domain, desc);
-          fprintf (out, "%s:%s\n",
-                   gc_component[idx].name,  my_percent_escape (desc));
+          for (backend = 0; backend < GC_BACKEND_NR; backend++)
+            backend_seen[backend] = 0;
+
+          pgmname = "";
+          for (; option && option->name; option++)
+            {
+              if ((option->flags & GC_OPT_FLAG_GROUP))
+                continue;
+              backend = option->backend;
+              if (backend_seen[backend])
+                continue;
+              backend_seen[backend] = 1;
+              assert (backend != GC_BACKEND_ANY);
+              if (gc_backend[backend].program
+                  && !gc_backend[backend].module_name)
+                continue;
+              pgmname = gnupg_module_name (gc_backend[backend].module_name);
+              break;
+            }
+
+          desc = gc_component[component].desc;
+          desc = my_dgettext (gc_component[component].desc_domain, desc);
+          fprintf (out, "%s:%s:",
+                   gc_component[component].name,  my_percent_escape (desc));
+          fprintf (out, "%s:\n",  my_percent_escape (pgmname));
         }
     }
 }
 
+
+
+static int
+all_digits_p (const char *p, size_t len)
+{
+  if (!len)
+    return 0; /* No. */
+  for (; len; len--, p++)
+    if (!isascii (*p) || !isdigit (*p))
+      return 0; /* No.  */
+  return 1; /* Yes.  */
+}
+
+
+/* Collect all error lines from file descriptor FD. Only lines
+   prefixed with TAG are considered.  Close that file descriptor
+   then.  Returns a list of error line items (which may be empty).
+   There is no error return.  */
+static error_line_t
+collect_error_output (int fd, const char *tag)
+{
+  FILE *fp;
+  char buffer[1024];
+  char *p, *p2, *p3;
+  int c, cont_line;
+  unsigned int pos;
+  error_line_t eitem, errlines, *errlines_tail;
+  size_t taglen = strlen (tag);
+
+  fp = fdopen (fd, "r");
+  if (!fp)
+    gc_error (1, errno, "can't fdopen pipe for reading");
+
+  errlines = NULL;
+  errlines_tail = &errlines;
+  pos = 0;
+  cont_line = 0;
+  while ((c=getc (fp)) != EOF)
+    {
+      buffer[pos++] = c;
+      if (pos >= sizeof buffer - 5 || c == '\n')
+        {
+          buffer[pos - (c == '\n')] = 0;
+          if (cont_line)
+            ; /*Ignore continuations of previous line. */
+          else if (!strncmp (buffer, tag, taglen) && buffer[taglen] == ':') 
+            {
+              /* "gpgsm: foo:4: bla" */
+              /* Yep, we are interested in this line.  */
+              p = buffer + taglen + 1;
+              while (*p == ' ' || *p == '\t')
+                p++;
+              if (!*p)
+                ; /* Empty lines are ignored.  */
+              else if ( (p2 = strchr (p, ':')) && (p3 = strchr (p2+1, ':'))
+                        && all_digits_p (p2+1, p3 - (p2+1)))
+                {
+                  /* Line in standard compiler format.  */
+                  p3++;
+                  while (*p3 == ' ' || *p3 == '\t')
+                    p3++;
+                  eitem = xmalloc (sizeof *eitem + strlen (p));
+                  eitem->next = NULL;
+                  strcpy (eitem->buffer, p);
+                  eitem->fname = eitem->buffer;
+                  eitem->buffer[p2-p] = 0;
+                  eitem->errtext = eitem->buffer + (p3 - p);
+                  /* (we already checked that there are only ascii
+                     digits followed by a colon) */
+                  eitem->lineno = 0;
+                  for (p2++; isdigit (*p2); p2++)
+                    eitem->lineno = eitem->lineno*10 + (*p2 - '0');
+                  *errlines_tail = eitem;
+                  errlines_tail = &eitem->next;
+                }
+              else
+                {
+                  /* Other error output.  */
+                  eitem = xmalloc (sizeof *eitem + strlen (p));
+                  eitem->next = NULL;
+                  strcpy (eitem->buffer, p);
+                  eitem->fname = NULL;
+                  eitem->errtext = eitem->buffer;
+                  eitem->lineno = 0;
+                  *errlines_tail = eitem;
+                  errlines_tail = &eitem->next;
+                }
+            }
+          pos = 0;
+          /* If this was not a complete line mark that we are in a
+             continuation.  */
+          cont_line = (c != '\n');
+        }
+    }
+  
+  /* We ignore error lines not terminated by a LF.  */
+
+  fclose (fp);
+  return errlines;
+}
+
+
+
+/* Check all components that are available.  */
+void
+gc_component_check_programs (FILE *out)
+{
+  gpg_error_t err;
+  gc_component_t component;
+  unsigned int result;
+  int backend_seen[GC_BACKEND_NR];
+  gc_backend_t backend;
+  gc_option_t *option;
+  const char *desc;
+  const char *pgmname;
+  const char *argv[2];
+  pid_t pid;
+  int exitcode;
+  int filedes[2];
+  error_line_t errlines, errptr;
+
+  /* We use a temporary file to collect the error output.  It would be
+     better to use a pipe here but as of now we have no suitable
+     fucntion to create a portable pipe outside of exechelp.  Thus it
+     is easier to use the tempfile approach.  */
+  for (component = 0; component < GC_COMPONENT_NR; component++)
+    {
+      if (!gc_component[component].options)
+        continue;
+
+      for (backend = 0; backend < GC_BACKEND_NR; backend++)
+        backend_seen[backend] = 0;
+
+      option = gc_component[component].options;
+      for (; option && option->name; option++)
+        {
+          if ((option->flags & GC_OPT_FLAG_GROUP))
+            continue;
+          backend = option->backend;
+          if (backend_seen[backend])
+            continue;
+          backend_seen[backend] = 1;
+          assert (backend != GC_BACKEND_ANY);
+          if (!gc_backend[backend].program)
+            continue;
+          if (!gc_backend[backend].module_name)
+            continue;
+
+          pgmname = gnupg_module_name (gc_backend[backend].module_name);
+          argv[0] = "--gpgconf-test";
+          argv[1] = NULL;
+
+          err = gnupg_create_inbound_pipe (filedes);
+          if (err)
+            gc_error (1, 0, _("error creating a pipe: %s\n"), 
+                      gpg_strerror (err));
+
+          result = 0;
+          errlines = NULL;
+          if (gnupg_spawn_process_fd (pgmname, argv, -1, -1, filedes[1], &pid))
+            {
+              close (filedes[0]);
+              close (filedes[1]);
+              result |= 1; /* Program could not be run.  */
+            }
+          else 
+            {
+              close (filedes[1]);
+              errlines = collect_error_output (filedes[0], 
+                                               gc_component[component].name);
+              if (gnupg_wait_process (pgmname, pid, &exitcode))
+                {
+                  if (exitcode == -1)
+                    result |= 1; /* Program could not be run or it
+                                    terminated abnormally.  */
+                  result |= 2; /* Program returned an error.  */
+                }
+            }
+          
+          /* If the program could not be run, we can't tell whether
+             the config file is good.  */
+          if ((result&1))
+            result |= 2;  
+          
+          desc = gc_component[component].desc;
+          desc = my_dgettext (gc_component[component].desc_domain, desc);
+          fprintf (out, "%s:%s:",
+                   gc_component[component].name, my_percent_escape (desc));
+          fputs (my_percent_escape (pgmname), out);
+          fprintf (out, ":%d:%d:", !(result & 1), !(result & 2));
+          for (errptr = errlines; errptr; errptr = errptr->next)
+            {
+              if (errptr != errlines)
+                fputs ("\n:::::", out); /* Continuation line.  */
+              if (errptr->fname)
+                fputs (my_percent_escape (errptr->fname), out);
+              putc (':', out);
+              if (errptr->fname)
+                fprintf (out, "%u", errptr->lineno);
+              putc (':', out);
+              fputs (my_percent_escape (errptr->errtext), out);
+              putc (':', out);
+            }
+          putc ('\n', out);
+          
+          while (errlines)
+            {
+              error_line_t tmp = errlines->next;
+              xfree (errlines);
+              errlines = tmp;
+            }
+          break; /* Loop over options of this component  */
+        }
+    } 
+}
+
+
 
 /* Find the component with the name NAME.  Returns -1 if not
    found.  */
@@ -1340,7 +1633,10 @@ retrieve_options_from_program (gc_component_t component, gc_backend_t backend)
   FILE *config;
   char *config_pathname;
 
-  cmd_line = xasprintf ("%s --gpgconf-list", gc_backend[backend].program);
+  cmd_line = xasprintf ("%s --gpgconf-list", 
+                        gc_backend[backend].module_name ?
+                        gnupg_module_name (gc_backend[backend].module_name) :
+                        gc_backend[backend].program );
 
   config = popen (cmd_line, "r");
   if (!config)
@@ -1641,6 +1937,8 @@ gc_component_retrieve_options (int component)
   while (process_all && ++component < GC_COMPONENT_NR);
 
 }
+
+
 
 /* Perform a simple validity check based on the type.  Return in
    NEW_VALUE_NR the value of the number in NEW_VALUE if OPTION is of
